@@ -1,5 +1,8 @@
 pref("startup.homepage_override_url","http://%LOCALE%.www.mozilla.com/%LOCALE%/%APP%/%VERSION%/whatsnew/");
 pref("startup.homepage_welcome_url","http://%LOCALE%.www.mozilla.com/%LOCALE%/%APP%/%VERSION%/firstrun/");
+// Interval: Time between checks for a new version (in seconds)
+// nightly=6 hours, official=24 hours
+pref("app.update.interval", 86400);
 // URL user can browse to manually if for some reason all update installation
 // attempts fail.
 pref("app.update.url.manual", "http://%LOCALE%.www.mozilla.com/%LOCALE%/%APP%/");
@@ -10,8 +13,5 @@ pref("app.update.url.details", "http://%LOCALE%.www.mozilla.com/%LOCALE%/%APP%/r
 // Release notes URL
 pref("app.releaseNotesURL", "http://%LOCALE%.www.mozilla.com/%LOCALE%/%APP%/%VERSION%/releasenotes/");
 
-pref("browser.search.param.yahoo-fr", "moz2");
-pref("browser.search.param.yahoo-fr-cjkt", "moz2");
-pref("browser.search.param.yahoo-f-CN", "D3_g");
-
-pref("browser.EULA.override", false); // Do show EULA on first start
+pref("browser.search.param.yahoo-fr", "moz35");
+pref("browser.search.param.yahoo-fr-cjkt", "moz35");
